<?php

namespace Drupal\acquia_contenthub_publisher\Plugin\QueueWorker;

use Drupal\acquia_contenthub\Client\ClientFactory;
use Drupal\acquia_contenthub\EntityCDFSerializer;
use Drupal\acquia_contenthub_publisher\AcquiaContentHubPublisherEvents;
use Drupal\acquia_contenthub_publisher\Event\ContentHubPublishEntitiesEvent;
use Drupal\acquia_contenthub_publisher\PublisherTracker;
use Drupal\Component\Utility\NestedArray;
use Drupal\Core\Entity\EntityTypeManagerInterface;
use Drupal\Core\Plugin\ContainerFactoryPluginInterface;
use Drupal\Core\Queue\QueueWorkerBase;
use Drupal\depcalc\DependencyCalculator;
use Drupal\depcalc\DependencyStack;
use Drupal\depcalc\DependentEntityWrapper;
use Symfony\Component\DependencyInjection\ContainerInterface;
use Symfony\Component\EventDispatcher\EventDispatcherInterface;

/**
 * @QueueWorker(
 *   id = "acquia_contenthub_publish_export",
 *   title = "Queue Worker to export entities to contenthub."
 * )
 */
class ContentHubExportQueueWorker extends QueueWorkerBase implements ContainerFactoryPluginInterface {

  /**
   * The entity type manager.
   *
   * @var \Drupal\Core\Entity\EntityTypeManagerInterface
   */
  protected $entityTypeManager;

  /**
   * The dependency serializer.
   *
   * @var \Drupal\depcalc\DependencyCalculator
   */
  protected $calculator;

  /**
   * The CDF Serializer.
   *
   * @var \Drupal\acquia_contenthub\EntityCDFSerializer
   */
  protected $serializer;

  /**
   * The client factory.
   *
   * @var \Drupal\acquia_contenthub\Client\ClientFactory
   */
  protected $factory;

  /**
   * The published entity tracker.
   *
   * @var \Drupal\acquia_contenthub_publisher\PublisherTracker
   */
  protected $tracker;

  /**
   * The event dispatcher.
   *
   * @var \Symfony\Component\EventDispatcher\EventDispatcherInterface
   */
  protected $dispatcher;

  /**
   * ContentHubExportQueueWorker constructor.
   *
   * @param \Drupal\Core\Entity\EntityTypeManagerInterface $entity_type_manager
   *   The entity type manager.
   * @param \Drupal\depcalc\DependencyCalculator $calculator
   *   The dependency calculator.
   * @param \Drupal\acquia_contenthub\EntityCDFSerializer $serializer
   *   The cdf serializer.
   * @param \Drupal\acquia_contenthub\Client\ClientFactory $factory
   *   The client factory.
   * @param \Drupal\acquia_contenthub_publisher\PublisherTracker $tracker
   *   The published entity tracker.
   * @param \Symfony\Component\EventDispatcher\EventDispatcherInterface $dispatcher
   *   The event dispatcher.
   * @param array $configuration
   *   The plugin configuration.
   * @param $plugin_id
   *   The plugin id.
   * @param $plugin_definition
   *   The plugin definition.
   */
  public function __construct(EntityTypeManagerInterface $entity_type_manager, DependencyCalculator $calculator, EntityCDFSerializer $serializer, ClientFactory $factory, PublisherTracker $tracker, EventDispatcherInterface $dispatcher, array $configuration, $plugin_id, $plugin_definition) {
    $this->entityTypeManager = $entity_type_manager;
    $this->calculator = $calculator;
    $this->serializer = $serializer;
    $this->factory = $factory;
    $this->tracker = $tracker;
    $this->dispatcher = $dispatcher;
    parent::__construct($configuration, $plugin_id, $plugin_definition);
  }

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container, array $configuration, $plugin_id, $plugin_definition) {
    return new static(
      $container->get('entity_type.manager'),
      $container->get('entity.dependency.calculator'),
      $container->get('entity.cdf.serializer'),
      $container->get('acquia_contenthub.client.factory'),
      $container->get('acquia_contenthub_publisher.tracker'),
      $container->get('event_dispatcher'),
      $configuration,
      $plugin_id,
      $plugin_definition
    );
  }

  /**
   * {@inheritdoc}
   */
  public function processItem($data) {
    $client = $this->factory->getClient();
    $storage = $this->entityTypeManager->getStorage($data->type);

    $entity = reset($storage->loadByProperties(['uuid' => $data->uuid]));
    $wrapper = new DependentEntityWrapper($entity);
    $stack = new DependencyStack();
    $this->calculator->calculateDependencies($wrapper, $stack);
    /** @var \Drupal\depcalc\DependentEntityWrapper[] $entities */
    $entities = NestedArray::mergeDeep([$wrapper->getUuid() => $wrapper], $stack->getDependenciesByUuid($wrapper->getDependencies()));

    // Modify/Remove objects before publishing to ContentHub service.
    $event = new ContentHubPublishEntitiesEvent($data->uuid, ...array_values($entities));
    $this->dispatcher->dispatch(AcquiaContentHubPublisherEvents::PUBLISH_ENTITIES, $event);
    $entities = $event->getDependencies();

    $output = $this->serializer->serializeEntities(...array_values($entities));

    // ContentHub backend determines new or update on the PUT endpoint.
    $response = $client->putEntities(...$output);
    if ($response->getStatusCode() == 202) {
      foreach ($output as $item) {
        $wrapper = !empty($entities[$item->getUuid()]) ? $entities[$item->getUuid()] : NULL;
        if ($wrapper) {
          $this->tracker->track($wrapper->getEntity(), sha1(json_encode($wrapper->getEntity()->toArray())));
        }
      }
    }
  }

}
