<?php

namespace Drupal\acquia_contenthub_subscriber\Plugin\QueueWorker;

use Drupal\acquia_contenthub\Client\ClientFactory;
use Drupal\acquia_contenthub\EntityCDFSerializer;
use Drupal\acquia_contenthub_subscriber\SubscriberTracker;
use Drupal\Core\Plugin\ContainerFactoryPluginInterface;
use Drupal\Core\Queue\QueueWorkerBase;
use Drupal\depcalc\DependencyStack;
use Symfony\Component\DependencyInjection\ContainerInterface;

/**
 * @QueueWorker(
 *   id = "acquia_contenthub_subscriber_import",
 *   title = "Queue Worker to import entities from contenthub."
 * )
 */
class ContentHubImportQueueWorker extends QueueWorkerBase implements ContainerFactoryPluginInterface {

  /**
   * The CDF Serializer.
   *
   * @var \Drupal\acquia_contenthub\EntityCDFSerializer
   */
  protected $serializer;

  /**
   * The client factory.
   *
   * @var \Drupal\acquia_contenthub\Client\ClientFactory
   */
  protected $factory;

  /**
   * The subscriber tracker.
   *
   * @var \Drupal\acquia_contenthub_subscriber\SubscriberTracker
   */
  protected $tracker;

  /**
   * ContentHubExportQueueWorker constructor.
   *
   * @param \Drupal\acquia_contenthub\EntityCDFSerializer $serializer
   *   The cdf serializer.
   * @param \Drupal\acquia_contenthub\Client\ClientFactory $factory
   *   The client factory.
   * @param \Drupal\acquia_contenthub_subscriber\SubscriberTracker $tracker
   *   The published entity tracker.
   * @param array $configuration
   *   The plugin configuration.
   * @param $plugin_id
   *   The plugin id.
   * @param $plugin_definition
   *   The plugin definition.
   */
  public function __construct(EntityCDFSerializer $serializer, ClientFactory $factory, SubscriberTracker $tracker, array $configuration, $plugin_id, $plugin_definition) {
    $this->serializer = $serializer;
    $this->factory = $factory;
    $this->tracker = $tracker;
    parent::__construct($configuration, $plugin_id, $plugin_definition);
  }

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container, array $configuration, $plugin_id, $plugin_definition) {
    return new static(
      $container->get('entity.cdf.serializer'),
      $container->get('acquia_contenthub.client.factory'),
      $container->get('acquia_contenthub_subscriber.tracker'),
      $configuration,
      $plugin_id,
      $plugin_definition
    );
  }

  /**
   * Processes acquia_contenthub_subscriber_import queue items.
   *
   * @param mixed $data
   *   The data in the queue.
   *
   * @throws \Drupal\Core\Entity\EntityStorageException
   * @throws \Drupal\acquia_contenthub\Exception\ContentHubException
   */
  public function processItem($data) {
    $document = $this->factory->getClient()->getEntities(explode(', ', $data->uuids));
    $missing_dependencies = [];
    foreach ($document->getEntities() as $cdf) {
      // @todo add the hash to the CDF so that we can check it here to see if we need to update.
      foreach ($cdf->getDependencies() as $dependency) {
        // If the document doesn't have a version of this dependency, it might be missing.
        if (!$document->hasEntity($dependency)) {
          $missing_dependencies[] = $dependency;
        }
      }
    }
    // Retrieve missing dependencies.
    if ($missing_dependencies) {
      $document->mergeDocuments($this->factory->getClient()->getEntities($missing_dependencies));
    }
    $stack = new DependencyStack();
    $this->serializer->unserializeEntities($document, $stack);
  }

}
