<?php

namespace Drupal\acquia_lift_support\EventSubscriber\CDF;

use Acquia\ContentHubClient\Attribute\CDFDataAttribute;
use Acquia\ContentHubClient\CDFAttribute;
use Acquia\ContentHubClient\CDFObject;
use Drupal\acquia_contenthub\AcquiaContentHubEvents;
use Drupal\acquia_contenthub\Client\ClientFactory;
use Drupal\acquia_contenthub\Event\CreateCDFEntityEvent;
use Drupal\acquia_contenthub\Session\ContentHubUserSession;
use Drupal\Component\Uuid\UuidInterface;
use Drupal\Core\Block\BlockManagerInterface;
use Drupal\Core\Config\ConfigFactoryInterface;
use Drupal\Core\Entity\ContentEntityInterface;
use Drupal\Core\Entity\EntityTypeManagerInterface;
use Drupal\Core\Render\Element;
use Drupal\Core\Render\RendererInterface;
use Drupal\Core\Session\AccountSwitcherInterface;
use Drupal\Core\Url;
use Drupal\image\Entity\ImageStyle;
use Symfony\Component\EventDispatcher\EventSubscriberInterface;

class EntityRenderHandler implements EventSubscriberInterface {

  /**
   * The account switcher.
   *
   * @var \Drupal\Core\Session\AccountSwitcherInterface
   */
  protected $accountSwitcher;

  /**
   * The Acquia Lift entity configuration.
   *
   * @var \Drupal\Core\Config\ImmutableConfig
   */
  protected $config;

  /**
   * The origin uuid.
   *
   * @var string
   */
  protected $origin;

  /**
   * The renderer.
   *
   * @var \Drupal\Core\Render\RendererInterface
   */
  protected $renderer;

  /**
   * The entity type manager.
   *
   * @var \Drupal\Core\Entity\EntityTypeManagerInterface
   */
  protected $entityTypeManager;

  /**
   * The Block Manager.
   *
   * @var \Drupal\Core\Block\BlockManagerInterface
   */
  protected $blockManager;

  /**
   * The uuid generator.
   *
   * @var \Drupal\Component\Uuid\UuidInterface
   */
  protected $uuidGenerator;

  /**
   * @var \Drupal\Core\Session\UserSession
   */
  protected $renderUser;

  public function __construct(AccountSwitcherInterface $account_switcher, ConfigFactoryInterface $config_factory, RendererInterface $renderer, EntityTypeManagerInterface $entity_type_manager, BlockManagerInterface $block_manager, UuidInterface $uuid_generator, ClientFactory $client_factory) {
    $this->accountSwitcher = $account_switcher;
    $this->config = $config_factory->get('acquia_lift_support.entity_config');
    $this->origin = $client_factory->getSettings()->getUuid();
    $this->renderer = $renderer;
    $this->entityTypeManager = $entity_type_manager;
    $this->blockManager = $block_manager;
    $this->uuidGenerator = $uuid_generator;
  }

  /**
   * {@inheritdoc}
   */
  public static function getSubscribedEvents() {
    $events[AcquiaContentHubEvents::CREATE_CDF_OBJECT][] = ['onCreateCDF', 100];
    return $events;
  }

  public function onCreateCDF(CreateCDFEntityEvent $event) {
    $entity = $event->getEntity();
    if (!$entity instanceof ContentEntityInterface) {
      // @todo we should support config entity rendering too.
      return;
    }
    if ($view_modes = $this->config->get("view_modes.{$entity->getEntityTypeId()}.{$entity->bundle()}")) {
      foreach (array_keys($view_modes) as $view_mode) {
        // The preview image field setting is saved along side the view modes.
        // Don't process it as one.
        if ($view_mode == 'acquia_lift_preview_image') {
          continue;
        }
        $display = \Drupal::entityTypeManager()->getStorage('entity_view_mode')->load("{$entity->getEntityTypeId()}.$view_mode");
        foreach ($entity->getTranslationLanguages() as $language) {
          $entity = $entity->getTranslation($language->getId());
          $cdf = new CDFObject('rendered_entity', $this->uuidGenerator->generate(), date('c'), date('c'), $this->origin);
          $elements = $this->getViewModeMinimalHtml($entity, $view_mode);
          $html = $this->renderer->renderPlain($elements);
          $cdf->addAttribute('data', CDFDataAttribute::TYPE_STRING, $html, 'und', CDFDataAttribute::class);
          $cdf->addAttribute('source_entity', CDFAttribute::TYPE_STRING, $entity->uuid());
          $cdf->addAttribute('label', CDFAttribute::TYPE_ARRAY_STRING, $entity->label(), $entity->language()->getId());
          $cdf->addAttribute('language', CDFAttribute::TYPE_STRING, $language->getId());
          $cdf->addAttribute('language_label', CDFAttribute::TYPE_STRING, $language->getName());
          $cdf->addAttribute('view_mode', CDFAttribute::TYPE_STRING, $view_mode);
          $cdf->addAttribute('view_mode_label', CDFAttribute::TYPE_STRING, $display->label());
          $url = Url::fromUserInput("/", ['absolute' => TRUE]);
          $cdf->addAttribute('base_url', CDFAttribute::TYPE_STRING, $url->toString());
          if (isset($view_modes['acquia_lift_preview_image'])) {
            $src = ImageStyle::load('acquia_lift_support_preview_image')->buildUrl($entity->{$view_modes['acquia_lift_preview_image']}->first()->entity->getFileUri());
            $cdf->addAttribute('preview_image', CDFAttribute::TYPE_STRING, $src);
          }
          $event->addCDF($cdf);
          // Document on the structured cdf if present.
          if ($source_entity_cdf = $event->getCDF($entity->uuid())) {
            if (!$attribute = $source_entity_cdf->getAttribute('renders')) {
              $source_entity_cdf->addAttribute('renders', CDFAttribute::TYPE_OBJECT);
              $attribute = $source_entity_cdf->getAttribute('renders');
            }
            else {
              $attribute = $source_entity_cdf->getAttribute('renders');
            }
            $value = $attribute->getValue();
            $value[$language->getId()][$view_mode] = $cdf->getUuid();
            $attribute->setValue($value[$language->getId()], $language->getId());
          }
        }
      }
    }
  }

  protected function getRenderUser() {
    if (!$this->renderUser) {
      $this->renderUser = new ContentHubUserSession($this->config->get('render_role'));
    }
    return $this->renderUser;
  }

  protected function getViewModeMinimalHtml(ContentEntityInterface $object, $view_mode) {
    // Creating a fake user account to give as context to the normalization.
    $account = $this->getRenderUser();
    // Checking for entity access permission to this particular account.
    $entity_access = $object->access('view', $account, TRUE);
    if (!$entity_access->isAllowed()) {
      return [];
    }
    $this->accountSwitcher->switchTo($account);
    // Render View Mode.
    $entity_type_id = $object->getEntityTypeId();
    // @todo allow different entity types to specify how to do this.
    if ($entity_type_id === 'block_content') {
      $build = $this->getBlockMinimalBuildArray($object, $view_mode);
    }
    else {
      $build = $this->entityTypeManager->getViewBuilder($entity_type_id)
        ->view($object, $view_mode, $object->language()->getId());
    }
    // Restore user account.
    $this->accountSwitcher->switchBack();
    return $build;
  }

  /**
   * Renders block using BlockViewBuilder.
   *
   * @param \Drupal\Core\Entity\ContentEntityInterface $object
   *   The Content Entity Object.
   * @param string $view_mode
   *   The request view mode identifier.
   *
   * @return array
   *   Render array for the block.
   */
  protected function getBlockMinimalBuildArray(ContentEntityInterface $object, $view_mode) {
    /** @var \Drupal\block_content\Plugin\Block\BlockContentBlock $block */
    $block = $this->blockManager->createInstance('block_content:' . $object->uuid());
    $build = [
      '#theme' => 'block',
      '#attributes' => [],
      '#contextual_links' => [],
      '#weight' => 0,
      '#configuration' => $block->getConfiguration(),
      '#plugin_id' => $block->getPluginId(),
      '#base_plugin_id' => $block->getBaseId(),
      '#derivative_plugin_id' => $block->getDerivativeId(),
    ];
    // Label controlled by the block__block_content__acquia_contenthub template
    // (hidden by default). Override the template in your theme to render a
    // block label.
    if ($build['#configuration']['label'] === '') {
      $build['#configuration']['label'] = $block->label();
    }
    // Block entity itself doesn't have configuration.
    $block->setConfigurationValue('view_mode', $view_mode);
    $build['#configuration']['view_mode'] = $view_mode;
    // See \Drupal\block\BlockViewBuilder::preRender() for reference.
    $content = $block->build();
    if ($content !== NULL && !Element::isEmpty($content)) {
      foreach (['#attributes', '#contextual_links'] as $property) {
        if (isset($content[$property])) {
          $build[$property] += $content[$property];
          unset($content[$property]);
        }
      }
    }
    $build['content'] = $content;
    return $build;
  }

}