<?php

namespace Drupal\acquia_contenthub;

/**
 * Defines events for the acquia_contenthub module.
 *
 * @see \Drupal\acquia_contenthub\Event\SerializeCDFEntityField
 * @see \Drupal\acquia_contenthub\Event\UnserializeCDFAttributeEvent
 * @see \Drupal\acquia_contenthub\Event\EntityDataTamperEvent
 * @see \Drupal\acquia_contenthub\Event\EntityImportEvent
 */
final class AcquiaContentHubEvents {

  /**
   * The event fired to collect ContentHub settings.
   *
   * ContentHub's settings can be provided in many different ways. This event
   * allows modules to provide these settings through various means.
   */
  const GET_SETTINGS = 'acquia_contenthub_get_settings';

  /**
   * The event fired when an entity is being serialized to CDF.
   *
   * This event allows modules to collaborate on entity CDF serialization.
   * The event listener method receives a
   * \Drupal\acquia_contenthub_publisher\Event\SerializeCDFEntity instance
   * which allows modules to provide serialization logic for custom attributes
   * or additional serializations for a given entity beyond the normal
   * "structure" serializations.
   *
   * @Event
   *
   * @see \Drupal\acquia_contenthub\Event\CreateCDFEntityEvent
   * @see \Drupal\acquia_contenthub\EntityCDFSerializer::serializeEntities()
   *
   * @var string
   */
  const CREATE_CDF_OBJECT = 'acquia_contenthub_serialize_entity';

  const POPULATE_CDF_ATTRIBUTES = 'acquia_contenthub_populate_cdf_attributes';

  const PARSE_CDF = 'acquia_contenthub_parse_cdf';

  /**
   * The event fired when a content entity field is being serialized to CDF.
   *
   * This event allows modules to collaborate on entity field serialization.
   * The event listener method receives a
   * \Drupal\acquia_contenthub_publisher\Event\SerializeCDFEntityField instance
   * which allows modules to provide serialization logic for their own field or
   * entity types.
   *
   * @Event
   *
   * @see \Drupal\acquia_contenthub\Event\SerializeCDFEntityField
   * @see \Drupal\acquia_contenthub\EntityCDFSerializer::serializeEntities()
   *
   * @var string
   */
  const SERIALIZE_CONTENT_ENTITY_FIELD = 'serialize_content_entity_field';


  /**
   * Name of the event fired when a CDF based PHP array is being unserialized.
   *
   * This event allows modules to collaborate on CDF unserialization.
   * The event listener method receives a
   * \Drupal\acquia_contenthub_publisher\Event\UnserializeCDFAttributeEvent instance
   * which allows modules to provide unserialization logic for specific
   * attribute types.
   *
   * @Event
   *
   * @see \Drupal\acquia_contenthub\Event\UnserializeCDFAttributeEvent
   * @see \Drupal\acquia_contenthub\EntityCDFSerializer::unserializeEntities()
   *
   * @var string
   */
  const UNSERIALIZE_CDF_ATTRIBUTE = 'unserialize_cdf_attribute';

  /**
   * Name of the event fired before a CDF based PHP array is being unserialized.
   *
   * This event allows modules to manipulate the data of the entire CDF array
   * before being unserialized by the normal process.
   * The event listener method receives a
   * \Drupal\acquia_contenthub_publisher\Event\EntityDataTamperEvent instance
   * which allows modules to tamper with the entity data before its unserialed.
   *
   * @Event
   *
   * @see \Drupal\acquia_contenthub\Event\EntityDataTamperEvent
   * @see \Drupal\acquia_contenthub\EntityCDFSerializer::unserializeEntities()
   *
   * @var string
   */
  const ENTITY_DATA_TAMPER = 'entity_data_tamper';

  /**
   * Name of the event fired after a CDF entity has been first saved.
   *
   * This event allows modules to respond to the saving of new entities after
   * they've been imported.
   * The event listener method receives a
   * \Drupal\acquia_contenthub_publisher\Event\EntityImportEvent instance which
   * allows modules to perform post-save events that might normally
   * happen (for instance) in a form submission.
   *
   * @Event
   *
   * @see \Drupal\acquia_contenthub\Event\EntityImportEvent
   *
   * @var string
   */
  const ENTITY_IMPORT_NEW = 'entity_import_new';

  /**
   * Name of the event fired after a CDF entity has been updated.
   *
   * This event allows modules to respond to the saving of entities after
   * they've been imported.
   * The event listener method receives a
   * \Drupal\acquia_contenthub_publisher\Event\EntityImportEvent instance which
   * allows modules to perform post-save events that might normally
   * happen (for instance) in a form submission.
   *
   * @Event
   *
   * @see \Drupal\acquia_contenthub\Event\EntityImportEvent
   *
   * @var string
   */
  const ENTITY_IMPORT_UPDATE = 'entity_import_update';

  const PRUNE_CDF = 'prune_cdf';

  const HANDLE_WEBHOOK = 'acquia_contenthub_handle_webhook';

  const IMPORT_FAILURE = 'acquia_contenthub_import_failure';

}
