<?php

namespace Drupal\acquia_contenthub\Client;

use Acquia\ContentHubClient\ContentHubClient;
use Acquia\Hmac\KeyLoader;
use Acquia\Hmac\Exception\KeyNotFoundException;
use Acquia\Hmac\RequestAuthenticator;
use Acquia\Hmac\AuthorizationHeader;
use Drupal\acquia_contenthub\AcquiaContentHubEvents;
use Drupal\acquia_contenthub\Event\AcquiaContentHubSettingsEvent;
use Drupal\Core\Logger\LoggerChannelFactoryInterface;
use Symfony\Bridge\PsrHttpMessage\Factory\DiactorosFactory;
use Symfony\Component\HttpFoundation\Request;
use Symfony\Component\EventDispatcher\EventDispatcherInterface;
use Drupal\Component\Uuid\Uuid;

/**
 * Instantiates an Acquia ContentHub Client object.
 *
 * @see \Acquia\ContentHubClient\ContentHub
 *
 */
class ClientFactory {

  /**
   * The event dispatcher.
   *
   * @var \Symfony\Component\EventDispatcher\EventDispatcherInterface
   */
  protected $dispatcher;

  /**
   * The contenthub client object.
   *
   * @var \Acquia\ContentHubClient\ContentHubClient
   */
  protected $client;

  /**
   * Settings Provider.
   */
  protected $settingsProvider;

  /**
   * Settings object.
   *
   * @var \Acquia\ContentHubClient\Settings
   */
  protected $settings;

  /**
   * Logger Factory.
   *
   * @var \Drupal\Core\Logger\LoggerChannelFactoryInterface
   */
  protected $loggerFactory;

  /**
   * ClientManagerFactory constructor.
   *
   * @param \Symfony\Component\EventDispatcher\EventDispatcherInterface $dispatcher
   *   The event dispatcher.
   * @param \Drupal\Core\Logger\LoggerChannelFactoryInterface $logger_factory
   */
  public function __construct(EventDispatcherInterface $dispatcher, LoggerChannelFactoryInterface $logger_factory) {
    $this->dispatcher = $dispatcher;
    $this->loggerFactory = $logger_factory;

    // Whenever a new client is constructed, make sure settings are invoked.
    $this->populateSettings();
  }

  /**
   * Call the event to populate contenthub settings.
   *
   */
  protected function populateSettings() {
    $event = new AcquiaContentHubSettingsEvent();
    $this->dispatcher->dispatch(AcquiaContentHubEvents::GET_SETTINGS, $event);
    $this->settings = $event->getSettings();
    $this->settingsProvider = $event->getProvider();
  }

  /**
   * Instantiates the content hub client.
   *
   * @return \Acquia\ContentHubClient\ContentHubClient | bool
   *   The ContentHub Client
   *
   * @throws \Drupal\acquia_contenthub\Exception\ContentHubException
   *   Throws exception when client cannot be instantiated.
   */
  public function getClient() {
    if (!$this->client) {
      if (!$this->settingsProvider) {
        $this->populateSettings();
      }

      // If any of these variables is empty, then we do NOT have a valid
      // connection.
      if (!$this->settings || !Uuid::isValid($this->settings->getUuid()) || empty($this->settings->getName()) || empty($this->settings->getUrl()) || empty($this->settings->getApiKey()) || empty($this->settings->getSecretKey())) {
        return FALSE;
      }
      
      // Find out the module version in use.
      $module_info = system_get_info('module', 'acquia_contenthub');
      $module_version = (isset($module_info['version'])) ? $module_info['version'] : '0.0.0';
      $drupal_version = (isset($module_info['core'])) ? $module_info['core'] : '0.0.0';
      $client_user_agent = 'AcquiaContentHub/' . $drupal_version . '-' . $module_version;

      // Override configuration.
      $config = [
        'base_url' => $this->settings->getUrl(),
        'client-user-agent' => $client_user_agent,
      ];
      
      $this->client = new ContentHubClient($config, $this->loggerFactory->get('acquia_contenthub'), $this->settings, $this->settings->getMiddleware());
    }
    return $this->client;
  }

  /**
   * Gets the settings provider from the settings event for contenthub settings.
   *
   * @return string
   */
  public function getProvider() {
    return $this->settingsProvider;
  }

  /**
   * Returns a settings object containing CH credentials and other related info.
   * @return \Acquia\ContentHubClient\Settings
   */
  public function getSettings() {
    return $this->settings;
  }

  /**
   * Make a call to get a client response based on the client name. If something
   * is returned, then access to contenthub was granted.
   *
   * Note, this receives a Symfony request, but uses a PSR7 Request to Auth.
   *
   * @param \Symfony\Component\HttpFoundation\Request
   *
   * @return bool | \Acquia\Hmac\KeyInterface
   */
  public function authenticate(Request $request) {
    if (!$this->client) {
      $this->getClient();
    }
    // Probably a better way to do this, but if one isn't created yet, bail.
    if (!$this->client) {
      return FALSE;
    }
    $keys = [
      $this->client->getSettings()->getApiKey() => $this->client->getSettings()->getSecretKey(),
      "Webhook" => $this->client->getSettings()->getSharedSecret()
    ];
    $keyLoader = new KeyLoader($keys);

    $authenticator = new RequestAuthenticator($keyLoader);

    // Authentication requires a PSR7 compatible request.
    $psr7Factory = new DiactorosFactory();
    $psr7_request = $psr7Factory->createRequest($request);

    try {
      return $authenticator->authenticate($psr7_request);
    }
    catch (KeyNotFoundException $exception) {
      $this->loggerFactory->get('acquia_contenthub')->debug('HMAC validation failed. [authorization = %authorization]. [authorization_header = %authorization_header]', [
        '%authorization' => '',
        '%authorization_header' => $request->headers->get('authorization'),
      ]);

      return FALSE;
    }
    return FALSE;
  }

  /**
   * @param \Symfony\Component\HttpFoundation\Request
   *
   * @return string
   */
  public function getSignature(Request $request) {
    $psr7Factory = new DiactorosFactory();
    $psr7_request = $psr7Factory->createRequest($request);

    $authHeader = AuthorizationHeader::createFromRequest($psr7_request);
    return $authHeader->getSignature();
  }
}
