<?php

namespace Drupal\acquia_contenthub\Event;

use Acquia\ContentHubClient\CDFObject;
use Drupal\Core\Entity\ContentEntityInterface;
use Drupal\Core\Field\FieldItemListInterface;
use Symfony\Component\EventDispatcher\Event;

/**
 * Event fired when serializing individual entity fields to CDF format.
 *
 * Subscribers to this event should manipulate $this->fieldData or
 * $this->return as necessary in order place values in the proper array format
 * for the CDF serialization process. An empty $this->fieldData will unset the
 * key/value from the list of CDF attributes.
 *
 * @see \Drupal\acquia_contenthub\AcquiaContentHubEvents
 */
class SerializeCDFEntityField extends Event {

  /**
   * The entity being serialized.
   *
   * @var \Drupal\Core\Entity\ContentEntityInterface
   */
  protected $entity;

  /**
   * The name of the field being serialized.
   *
   * @var string
   */
  protected $fieldName;

  /**
   * The field being serialized.
   *
   * @var \Drupal\Core\Field\FieldItemListInterface
   */
  protected $field;

  /**
   * The array of data to serialize for this field.
   *
   * @var array
   */
  protected $fieldData;

  /**
   * The main return object of the serialization process.
   *
   * @var \Acquia\ContentHubClient\CDFObject
   */
  protected $cdf;

  /**
   * SerializeCDFEntityField constructor.
   *
   * @param \Drupal\Core\Entity\ContentEntityInterface $entity
   *   The entity to which the field belongs.
   * @param string $field_name
   *   The name of the field.
   * @param \Drupal\Core\Field\FieldItemListInterface $field
   *   The field item list object.
   * @param \Acquia\ContentHubClient\CDFObject $cdf
   *   The CDF Object being created.
   */
  public function __construct(ContentEntityInterface $entity, $field_name, FieldItemListInterface $field, CDFObject $cdf) {
    $this->entity = $entity;
    $this->fieldName = $field_name;
    $this->field = $field;
    $this->cdf = $cdf;
  }

  /**
   * The entity to which the field belongs.
   *
   * @return \Drupal\Core\Entity\ContentEntityInterface
   */
  public function getEntity() {
    return $this->entity;
  }

  /**
   * The name of the field.
   *
   * @return string
   */
  public function getFieldName() {
    return $this->fieldName;
  }

  /**
   * The field item list object.
   *
   * @return \Drupal\Core\Field\FieldItemListInterface
   */
  public function getField() {
    return $this->field;
  }

  /**
   * The field item list object for a particular language.
   *
   * @param string $langcode
   *   The language of the field value to get.
   *
   * @return \Drupal\Core\Field\FieldItemListInterface
   */
  public function getFieldTranslation($langcode) {
    $entity = $this->getEntity()->getTranslation($langcode);
    return $entity->{$this->getField()->getName()};
  }

  /**
   * The field data to serialize.
   *
   * @return array
   */
  public function getFieldData() {
    return $this->fieldData;
  }

  /**
   * Set the field data to serialize.
   *
   * @param array $data
   */
  public function setFieldData(array $data) {
    $this->fieldData = $data;
  }

  /**
   * The CDF Object.
   *
   * @return \Acquia\ContentHubClient\CDFObject
   */
  public function getCDF() {
    return $this->cdf;
  }

}
