<?php

namespace Drupal\acquia_contenthub\EventSubscriber\CDF;

use Acquia\ContentHubClient\Attribute\CDFDataAttribute;
use Acquia\ContentHubClient\CDFObject;
use Drupal\acquia_contenthub\AcquiaContentHubEvents;
use Drupal\acquia_contenthub\Client\ClientFactory;
use Drupal\acquia_contenthub\Event\CreateCDFEntityEvent;
use Drupal\acquia_contenthub\Event\ParseCDFEntityEvent;
use Drupal\Component\Utility\NestedArray;
use Drupal\Core\Config\ConfigFactoryInterface;
use Drupal\Core\Config\Entity\ConfigEntityInterface;
use Drupal\Core\Entity\EntityRepositoryInterface;
use Drupal\Core\Serialization\Yaml;
use Drupal\depcalc\DependencyCalculator;
use Symfony\Component\EventDispatcher\EventSubscriberInterface;

class ConfigEntityHandler implements EventSubscriberInterface {

  /**
   * The dependency calculator.
   *
   * @var \Drupal\depcalc\DependencyCalculator
   */
  protected $calculator;

  /**
   * The configuration factory.
   *
   * @var \Drupal\Core\Config\ConfigFactoryInterface
   */
  protected $configFactory;

  /**
   * The client factory.
   *
   * @var \Drupal\acquia_contenthub\Client\ClientFactory
   */
  protected $clientFactory;

  /**
   * ConfigEntityHandler constructor.
   *
   * @param \Drupal\depcalc\DependencyCalculator $calculator
   *   The dependency calculator.
   * @param \Drupal\Core\Config\ConfigFactoryInterface $config_factory
   *   The configuration factory.
   * @param \Drupal\acquia_contenthub\Client\ClientFactory $factory
   *   The client factory.
   */
  public function __construct(DependencyCalculator $calculator, ConfigFactoryInterface $config_factory, ClientFactory $factory) {
    $this->calculator = $calculator;
    $this->configFactory = $config_factory;
    $this->clientFactory = $factory;
  }

  /**
   * {@inheritdoc}
   */
  public static function getSubscribedEvents() {
    $events[AcquiaContentHubEvents::CREATE_CDF_OBJECT][] = ['onCreateCDF', 100];
    $events[AcquiaContentHubEvents::PARSE_CDF][] = ['onParseCDF', 100];
    return $events;
  }

  /**
   * {@inheritdoc}
   */
  public function onCreateCDF(CreateCDFEntityEvent $event) {
    $entity = $event->getEntity();
    if (!$entity instanceof ConfigEntityInterface) {
      // Bail early if this isn't a config entity.
      return;
    }

    $settings = $this->clientFactory->getSettings();

    $cdf = new CDFObject('drupal8_config_entity', $entity->uuid(), date('c'), date('c'), $settings->getUuid());
    $metadata = [
      'default_language' => $entity->language()->getId(),
    ];
    if ($dependencies = $event->getDependencies()) {
      $metadata['dependencies'] = $dependencies;
    }
    // Some config entities don't have a dependency on their provider module.
    if ($entity->getEntityType()->getProvider() != 'core') {
      if (!empty($metadata['dependencies']['module'])) {
        $metadata['dependencies']['module'] = NestedArray::mergeDeep($metadata['dependencies']['module'], [
          $entity->getEntityType()->getProvider()
        ]);
      }
      else {
        $metadata['dependencies']['module'][] = $entity->getEntityType()->getProvider();
      }
    }
    $cdf->setMetadata($metadata);
    $idKey = "{$entity->getEntityType()->getConfigPrefix()}.{$entity->get($entity->getEntityType()->getKey('id'))}";
    $config = $this->configFactory->get($idKey);
    $cdf->addAttribute('data', CDFDataAttribute::TYPE_STRING, Yaml::encode($config->getRawData()), 'und', CDFDataAttribute::class);
    $event->addCDF($cdf);
  }

  /**
   * {@inheritdoc}
   */
  public function onParseCDF(ParseCDFEntityEvent $event) {
    $cdf = $event->getCDF();
    if ($cdf->getType() !== 'drupal8_config_entity') {
      // Bail early if this isn't a config entity.
      return;
    }

    foreach ($cdf->getAttribute('data')->getValue() as $langcode => $values) {
      $values = Yaml::decode($values);
      /** @var EntityRepositoryInterface $repository */
      $repository = \Drupal::service('entity.repository');
      // @todo Logic:
      // If entity by this uuid does not exist
      //   Check to see if entity by this id exists
      //   Check to see if that entity was previously imported
      //     if previously imported, throw some exception or dispatch an event to resolve it.
      //   If nothing exists or conflicts, create a new entity.
      // If entity by this uuid DOES exists
      //   Update it.
      $entity_type_id = $cdf->getAttribute('entity_type')->getValue()['und'];
      /** @var ConfigEntityInterface $entity */
      if (!$entity = $repository->loadEntityByUuid($entity_type_id, $cdf->getUuid())) {
        $entity = \Drupal::entityTypeManager()->getStorage($entity_type_id)->createFromStorageRecord($values);
        // If there's an existing entity by the same id, update it.
        if ($old_entity = \Drupal::entityTypeManager()->getStorage($entity_type_id)->load($entity->id())) {
          // @todo check if this entity was previously imported. (Multiple entities of the same ID from different publishers)
          $values['uuid'] = $old_entity->uuid();
          /** @var ConfigEntityInterface $old_entity */
          $old_entity = \Drupal::entityTypeManager()->getStorage($entity_type_id)->createFromStorageRecord($values);
          $old_entity->enforceIsNew(FALSE);
          $event->setEntity($old_entity);
          return;
        }
      }
      else {
        $entity = \Drupal::entityTypeManager()->getStorage($entity_type_id)->createFromStorageRecord($values);
        $entity->enforceIsNew(FALSE);
      }
      $event->setEntity($entity);
    }
  }

}
