<?php

namespace Drupal\acquia_contenthub\EventSubscriber\CDF;

use Acquia\ContentHubClient\Attribute\CDFDataAttribute;
use Acquia\ContentHubClient\CDFObject;
use Drupal\acquia_contenthub\AcquiaContentHubEvents;
use Drupal\acquia_contenthub\Client\ClientFactory;
use Drupal\acquia_contenthub\Event\CreateCDFEntityEvent;
use Drupal\acquia_contenthub\Event\SerializeCDFEntityField;
use Drupal\acquia_contenthub\Event\UnserializeCDFAttributeEvent;
use Drupal\acquia_contenthub\Event\ParseCDFEntityEvent;
use Drupal\Component\Utility\NestedArray;
use Drupal\Core\Config\ConfigFactoryInterface;
use Drupal\Core\Entity\ContentEntityInterface;
use Drupal\Core\TypedData\TranslatableInterface;
use Drupal\depcalc\DependencyCalculator;
use Symfony\Component\EventDispatcher\EventDispatcherInterface;
use Symfony\Component\EventDispatcher\EventSubscriberInterface;

class ContentEntityHandler implements EventSubscriberInterface {

  /**
   * The dependency calculator.
   *
   * @var \Drupal\depcalc\DependencyCalculator
   */
  protected $calculator;

  /**
   * The configuration factory.
   *
   * @var \Drupal\Core\Config\ConfigFactoryInterface
   */
  protected $configFactory;

  /**
   * The event dispatcher.
   *
   * @var \Symfony\Component\EventDispatcher\EventDispatcherInterface
   */
  protected $dispatcher;

  /**
   * The client factory.
   *
   * @var \Drupal\acquia_contenthub\Client\ClientFactory
   */
  protected $clientFactory;

  /**
   * ContentEntity constructor.
   *
   * @param DependencyCalculator $calculator
   *   The dependency calculator.
   * @param ConfigFactoryInterface $config_factory
   *   The configuration factory.
   * @param EventDispatcherInterface $dispatcher
   *   The event dispatcher.
   * @param \Drupal\acquia_contenthub\Client\ClientFactory $factory
   *   The client factory.
   */
  public function __construct(DependencyCalculator $calculator, ConfigFactoryInterface $config_factory, EventDispatcherInterface $dispatcher, ClientFactory $factory) {
    $this->calculator = $calculator;
    $this->configFactory = $config_factory;
    $this->dispatcher = $dispatcher;
    $this->clientFactory = $factory;
  }

  /**
   * {@inheritdoc}
   */
  public static function getSubscribedEvents() {
    $events[AcquiaContentHubEvents::CREATE_CDF_OBJECT][] = ['onCreateCDF', 100];
    $events[AcquiaContentHubEvents::PARSE_CDF][] = ['onParseCDF', 100];
    return $events;
  }

  /**
   * {@inheritdoc}
   */
  public function onCreateCDF(CreateCDFEntityEvent $event) {
    $entity = $event->getEntity();
    if (!$entity instanceof ContentEntityInterface) {
      // Bail early if this isn't a content entity.
      return;
    }

    $settings = $this->clientFactory->getSettings();

    $cdf = new CDFObject('drupal8_content_entity', $entity->uuid(), date('c'), date('c'), $settings->getUuid());
    $metadata = [
      'default_language' => $entity->language()->getId(),
    ];
    if ($dependencies = $event->getDependencies()) {
      $metadata['dependencies'] = $dependencies;
    }
    $cdf->setMetadata($metadata);

    $fields = [];
    foreach ($entity as $field_name => $field) {
      $fieldEvent = new SerializeCDFEntityField($entity, $field_name, $field, $cdf);
      $this->dispatcher->dispatch(AcquiaContentHubEvents::SERIALIZE_CONTENT_ENTITY_FIELD, $fieldEvent);
      if ($data = $fieldEvent->getFieldData()) {
        $fields[$field_name] = $data;
      }
    }
    $cdf->addAttribute('data', CDFDataAttribute::TYPE_STRING, json_encode($fields), 'und', CDFDataAttribute::class);
    $event->addCDF($cdf);
  }

  /**
   * {@inheritdoc}
   */
  public function onParseCDF(ParseCDFEntityEvent $event) {
    $cdf = $event->getCDF();
    if ($cdf->getType() !== 'drupal8_content_entity') {
      // Bail early if this isn't a content entity.
      return;
    }
    $entity_values = [];
    $entity_type_id = $cdf->getAttribute('entity_type')->getValue()['und'];
    $storage = $this->getEntityTypeManager()->getStorage($entity_type_id);
    $entity_type = $this->getEntityTypeManager()->getDefinition($entity_type_id);
    foreach (json_decode($cdf->getAttribute('data')->getValue()['und'], TRUE) as $field_name => $attribute) {
      $unserialize_event = new UnserializeCDFAttributeEvent($entity_type, $field_name, $attribute, $cdf->getMetadata()['field'][$field_name], $event->getStack());
      $this->dispatcher->dispatch(AcquiaContentHubEvents::UNSERIALIZE_CDF_ATTRIBUTE, $unserialize_event);
      $value = $unserialize_event->getValue();
      if ($value) {
        $entity_values = NestedArray::mergeDeep($entity_values, $value);
      }
    }

    if (!$entity = $this->getEntityRepository()->loadEntityByUuid($entity_type_id, $cdf->getUuid())) {
      $entity = $storage->create($entity_values[$cdf->getMetadata()['default_language']]);
    }
    if ($entity instanceof TranslatableInterface) {
      foreach ($entity_values as $langcode => $values) {
        if (!$entity->hasTranslation($langcode)) {
          $entity->addTranslation($langcode, $values);
        }
        else {
          /** @var \Drupal\Core\Entity\ContentEntityInterface $entity */
          $entity = $entity->getTranslation($langcode);
          foreach ($entity as $field_name => $field) {
            if (!empty($values[$field_name])) {
              $entity->set($field_name, $values[$field_name]);
            }
          }
        }
      }
    }
    $event->setEntity($entity);
  }

  /**
   * Get the entity type manager.
   *
   * @return \Drupal\Core\Entity\EntityTypeManagerInterface
   */
  protected function getEntityTypeManager() {
    return \Drupal::entityTypeManager();
  }

  /**
   * Get the entity type repository.
   *
   * @return \Drupal\Core\Entity\EntityTypeRepositoryInterface
   */
  protected function getEntityRepository() {
    return \Drupal::service('entity.repository');
  }

}
