<?php

namespace Drupal\acquia_contenthub\Form;

use Acquia\ContentHubClient\ContentHubClient;
use Acquia\ContentHubClient\Settings;
use Drupal\acquia_contenthub\Client\ClientFactory;
use Drupal\acquia_contenthub\Client\ClientManagerInterface;
use Drupal\Core\Form\FormStateInterface;
use Drupal\Core\Form\ConfigFormBase;
use Drupal\Core\Logger\LoggerChannelFactoryInterface;
use Symfony\Component\DependencyInjection\ContainerInterface;
use Drupal\Component\Utility\UrlHelper;
use Drupal\Component\Uuid\Uuid;
use Drupal\acquia_contenthub\ContentHubSubscription;
use Drupal\Core\Config\ConfigFactoryInterface;

/**
 * Defines the form to configure the Content Hub connection settings.
 *
 * @todo This is part of the settings work and needs a proper plan.
 */
class ContentHubSettingsForm extends ConfigFormBase {

  /**
   * The entity manager.
   *
   * @var |Drupal\acquia_contenthub\Client\ClientFactory
   */
  protected $clientFactory;

  protected $loggerFactory;

  /**
   * Contenthub Settings
   *
   * @var \Acquia\ContentHubClient\Settings
   */
  protected $settings;

  /**
   * Config Factory.
   *
   * @var \Drupal\Core\Config\ConfigFactoryInterface
   */

  protected $configFactory;

  /**
   * {@inheritdoc}
   */
  public function getFormId() {
    return 'acquia_contenthub.admin_settings';
  }

  /**
   * {@inheritdoc}
   */
  protected function getEditableConfigNames() {
    return ['acquia_contenthub.admin_settings'];
  }

  /**
   * ContentHubSettingsForm constructor.
   *
   * @param \Drupal\acquia_contenthub\Client\ClientManagerInterface $client_manager
   *   The client manager.
   * @param \Drupal\acquia_contenthub\ContentHubSubscription $contenthub_subscription
   *   The content hub subscription.
   * @param \Drupal\Core\Config\ConfigFactoryInterface $config_factory
   *   The config factory.
   */
  public function __construct(ConfigFactoryInterface $config_factory, ClientFactory $client_factory, LoggerChannelFactoryInterface $logger_factory) {
    $this->configFactory = $config_factory;

    /* The client manager might be null but is still good for later */
    $this->clientFactory = $client_factory;

    $this->loggerFactory = $logger_factory;
  }

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container) {
    return new static(
      $container->get('config.factory'),
      $container->get('acquia_contenthub.client.factory'),
      $container->get('logger.factory')
    );
  }

  /**
   * {@inheritdoc}
   */
  public function buildForm(array $form, FormStateInterface $form_state) {

    $provider = $this->clientFactory->getProvider();
    $disabled = $provider != 'core_config';
    $settings = $this->clientFactory->getSettings();

    if ($disabled) {
      drupal_set_message($this->t("Settings are being provided by %provider. Values shown for informational purposes only.", ['%provider' => $provider]));
    }

    $form['settings'] = [
      '#type' => 'fieldset',
      '#title' => $this->t('Connection Settings'),
      '#collapsible' => TRUE,
      '#description' => $this->t('Settings for connection to Acquia Content Hub'),
      '#disabled' => $disabled,
    ];

    $form['settings']['hostname'] = [
      '#type' => 'textfield',
      '#title' => $this->t('Acquia Content Hub Hostname'),
      '#description' => $this->t('The hostname of the Acquia Content Hub API without trailing slash at end of URL, e.g. http://localhost:5000'),
      '#default_value' => $settings->getUrl(),
      '#required' => TRUE,
    ];

    $form['settings']['api_key'] = [
      '#type' => 'textfield',
      '#title' => $this->t('API Key'),
      '#default_value' => $settings->getApiKey(),
      '#required' => TRUE,
    ];

    $form['settings']['secret_key'] = [
      '#type' => 'password',
      '#title' => $this->t('Secret Key'),
      '#default_value' => $settings->getSecretKey(),
    ];

    $client_name = $settings->getName();

    $form['settings']['client_name'] = [
      '#type' => 'textfield',
      '#title' => $this->t('Client Name'),
      '#default_value' => $client_name,
      '#required' => TRUE,
      '#description' => $this->t('A unique client name by which Acquia Content Hub will identify this site. The name of this client site cannot be changed once set.'),
      '#attributes' => empty($client_name) ? [] : ['readonly' => TRUE],
    ];

    $form['settings']['origin'] = [
      '#type' => 'item',
      '#title' => $this->t("Site's Origin UUID"),
      '#markup' => $settings->getUuid(),
    ];

    if (!$disabled) {
      if (!empty($settings->getUuid())) {
        $form['actions']['unregister'] = [
          '#type' => 'submit',
          '#value' => $this->t('Unregister Site'),
          '#button_type' => 'secondary',
          '#weight' => 999,
          '#submit' => ['::unregister'],

          // No validation at all is required in the equivocate case, so
          // we include this here to make it skip the form-level validator.
          '#validate' => [],
        ];
      }

      return parent::buildForm($form, $form_state);
    }

    // Just return the form if its not being used to save settings.
    return $form;
  }

  /**
   * {@inheritdoc}
   */
  public function validateForm(array &$form, FormStateInterface $form_state) {
    $hostname = NULL;
    if ($form_state->getTriggeringElement()['#id'] == 'edit-unregister') {
      return;
    }
    if (UrlHelper::isValid($form_state->getValue('hostname'), TRUE)) {
      // Remove trailing slash at end of URL.
      $hostname = rtrim($form_state->getValue('hostname'), '/');
    }
    else {
      return $form_state->setErrorByName('hostname', $this->t('This is not a valid URL. Please insert it again.'));
    }

    $api = NULL;
    // Important. This should never validate if it is an UUID. Lift 3 does not
    // use UUIDs for the api_key but they are valid for Content Hub.
    if ($form_state->getValue('api_key')) {
      $api = $form_state->getValue('api_key');
    }
    else {
      return $form_state->setErrorByName('api_key', $this->t('Please insert an API Key.'));
    }

    $secret = NULL;
    if ($form_state->hasValue('secret_key')) {
      $secret = $form_state->getValue('secret_key');
    }
    else {
      return $form_state->setErrorByName('secret_key', $this->t('Please insert a Secret Key.'));
    }

    if ($form_state->hasValue('client_name')) {
      $client_name = $form_state->getValue('client_name');
    }
    else {
      return $form_state->setErrorByName('client_name', $this->t('Please insert a Client Name.'));
    }

    if (Uuid::isValid($form_state->getValue('origin'))) {
      $origin = $form_state->getValue('origin');
    }
    else {
      $origin = '';
    }

    /* Validate that the client name does not exist yet. */
    if ($client_name == ContentHubClient::clientNameExists($client_name, $hostname, $api, $secret)) {
      $message = $this->t('The client name "%name" is already being used. Please insert another one.', [
        '%name' => $client_name,
      ]);
      return $form_state->setErrorByName('client_name', $message);
    }
  }

  public function unregister(array &$form, FormStateInterface $form_state) {
    /*
    $webhook_register = (bool) $this->config->get('webhook_uuid');
    $webhook_url = $this->config->get('webhook_url');
    // Un-register the webhook.
    if ($webhook_register) {
      $this->unregisterWebhook($webhook_url);
    }*/
    $client = $this->clientFactory->getClient();
    try {
      $client->deleteClient();
    }
    catch (\Exception $e) {
      $deleted = true;
    }

    $config = $this->config('acquia_contenthub.admin_settings');
    $client_name = $config->get('client_name');
    $config->delete();
    drupal_set_message($this->t("Successfully disconnected site %site from contenthub", ['%site' => $client_name]));
  }

  /**
   * {@inheritdoc}
   */
  public function submitForm(array &$form, FormStateInterface $form_state) {
    parent::submitForm($form, $form_state);

    // We assume here all inserted values have passed validation.
    // First Register the site to Content Hub.
    $client_name = $form_state->getValue('client_name');

    $logger_channel = $this->loggerFactory->get('acquia_contenthub');

    try {
      if ($client = ContentHubClient::register($logger_channel, $client_name, $form_state->getValue('hostname'), $form_state->getValue('api_key'), $form_state->getValue('secret_key'))) {
        // Registration was successful. Save the rest of the values.
        // Get the admin config.
        $config = $this->config('acquia_contenthub.admin_settings');
        $settings = $client->getSettings();
        if ($settings->getUrl()) {
          // Remove trailing slash at end of URL.
          $hostname = rtrim($settings->getUrl(), '/');
          $config->set('hostname', $hostname);
        }

        if ($settings->getApiKey()) {
          $config->set('api_key', $settings->getApiKey());
        }

        if ($settings->getSecretKey()) {
          $config->set('secret_key', $settings->getSecretKey());
        }

        if ($settings->getUuid()) {
          $config->set('origin', $settings->getUuid());
        }

        if ($settings->getName()) {
          $config->set('client_name', $settings->getName());
        }

        $remote = $client->getRemoteSettings();
        if ($remote['shared_secret']) {
          $config->set('shared_secret', $remote['shared_secret']);
        }

        $config->save();
      }
    } catch (\GuzzleHttp\Exception\RequestException $ch_error) {
      // Get the admin config.
      $config = $this->config('acquia_contenthub.admin_settings');

      // Call drupal_get_messages, to override the dsm. Otherwise,
      // on save it will show two messages.
      drupal_get_messages();
      if (!empty($config->get('origin'))) {
        drupal_set_message($this->t('Client is already registered with Acquia Content Hub.'), 'error');
      }
      else {
        drupal_set_message($this->t('There is a problem connecting to Acquia Content Hub. Please ensure that your hostname and credentials are correct.'), 'error');
      }
    }
  }
}
