<?php

namespace Drupal\acquia_contenthub\Form;

use Drupal\acquia_contenthub\Client\ClientFactory;
use Drupal\Core\Form\FormStateInterface;
use Drupal\Core\Form\ConfigFormBase;
use Symfony\Component\DependencyInjection\ContainerInterface;
use Drupal\Component\Utility\UrlHelper;

/**
 * Defines the form to register the webhooks.
 */
class WebhooksSettingsForm extends ConfigFormBase {

  /**
   * Config Factory.
   *
   * @var \Drupal\Core\Config\ConfigFactory
   */
  protected $configFactory;

  /**
   * Content Hub Subscription.
   *
   * @var \Drupal\acquia_contenthub\Client\ClientFactory
   */
  protected $clientFactory;

  /**
   * WebhooksSettingsForm constructor.
   *
   * @param \Drupal\acquia_contenthub\Client\ClientFactory $client_factory
   *   The client factory.
   */
  public function __construct(ClientFactory $client_factory) {
    $this->clientFactory = $client_factory;
  }

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container) {
    return new static(
      $container->get('acquia_contenthub.client.factory')
    );
  }

  /**
   * {@inheritdoc}
   */
  public function getFormId() {
    return 'acquia_contenthub.admin_settings';
  }

  /**
   * {@inheritdoc}
   */
  protected function getEditableConfigNames() {
    return ['acquia_contenthub.admin_settings'];
  }

  /**
   * {@inheritdoc}
   */
  public function buildForm(array $form, FormStateInterface $form_state) {
    $client = $this->clientFactory->getClient();
    $webhooks = $client->getWebHooks();

    $form['webhook_settings'] = [
      '#type' => 'fieldset',
      '#title' => $this->t('Administer Webhooks'),
      '#collapsible' => FALSE,
      '#description' => $this->t('Manage Acquia Content Hub Webhooks. To delete a webhook, simply remove the URL you wish to delete above.'),
    ];

    $form['webhook_settings']['webhooks'] = [
      '#type' => 'table',
      '#header' => [
        'uuid' => $this->t('UUID'),
        'url' => $this->t('URL'),
      ],
    ];

    foreach ($webhooks as $key=> $webhook) {
      $form['webhook_settings']['webhooks'][$key]['uuid'] = ['#markup' => $webhook['uuid']];
      $form['webhook_settings']['webhooks'][$key]['url'] = [
        '#type' => 'textfield',
        '#title' => '',
        '#title_display' => 'invisible',
        //it is important to use #name attribute
        // instead #default_value need to use #value
        '#default_value' => $webhook['url'],
      ];
    }
    // Add a new webhook field
    $form['webhook_settings']['webhooks'][] = [
      'uuid' => ['#markup' => 'Add a new Webhook:'],
      'url' => [
        '#type' => 'textfield',
        '#title' => '',
        '#title_display' => 'invisible',
      ],
    ];

    return parent::buildForm($form, $form_state);
  }

  /**
   * {@inheritdoc}
   */
  public function validateForm(array &$form, FormStateInterface $form_state) {
    // Prune the last webhook if it was empty
    $webhooks = $form_state->getValue('webhooks');
    foreach ($webhooks AS $webhook) {
      if (!empty($webhook['url'] && !UrlHelper::isValid($webhook['url'], TRUE))) {
        return $form_state->setErrorByName('webhook_url', $this->t('%webhook is not a valid URL. Please insert it again.', ["%webhook" => $webhook['url']]));
      }
    }
  }

  /**
   * {@inheritdoc}
   */
  public function submitForm(array &$form, FormStateInterface $form_state) {
    $webhooks = $form_state->getValue('webhooks');
    $new_webhook = array_pop($webhooks);
    $client = $this->clientFactory->getClient();
    $service_webhooks = $client->getWebHooks();

    $response = [];
    $updated = $deleted = $created = $error = $key = 0;
    foreach ($webhooks AS $key => $webhook) {
      if (empty($webhook['url'])) {
        $response[$key] = $client->deleteWebhook($service_webhooks[$key]['uuid']);
        $deleted++;
      }
      elseif ($webhook['url'] != $service_webhooks[$key]['url']) {
        /** @var \GuzzleHttp\Psr7\Response */
        $response[$key] = $client->updateWebhook($service_webhooks[$key]['uuid'], $webhook['url']);
        // Yay we recieved a response. Is it valid?
        if ($response[$key]) {
          if ($response[$key]->getStatusCode() == '200') {
            $updated++;
          }
          elseif ($response[$key]->getStatusCode() == '403') {
            drupal_set_message($this->t('Unable to update webhook %url, Permission denied', ['%url' => $webhook['url']]), 'error');
            $error++;
          }
          else {
            drupal_set_message($this->t('Unable to update webhook %url, Error %error', ['%url' => $webhook['url'], '%error' => $response[$key]->getStatusCode()]), 'error');
            $error++;
          }
        }
      }
    }

    // Process the new webhook at the end of the list.
    if (!empty($new_webhook['url'])) {
      $key++;
      $response[$key] = $client->addWebhook($new_webhook['url']);
      if (empty($response[$key]) || (isset($response[$key]['success']) && $response[$key]['success'] == FALSE)) {
        drupal_set_message($this->t("Unable to create URL %url, Error %error: ".$response[$key]['error']['message'], ['%url' => $new_webhook['url'], '%error' => $response[$key]['error']['code']]), 'error');
        $error++;
      }
      else {
        $created++;
      }
    }

    if (!$created && !$deleted && !$updated) {
      drupal_set_message($this->t('No updates to webhooks were made.'), 'warning');
    }
    elseif ($error > 0) {
      drupal_set_message($this->t('Only some webhooks were updated, check the error log for failures. %deleted were deleted, %updated were updated, and %created were created ', ['%deleted' => $deleted, '%updated' => $updated, '%created' => $created]), 'warning');
    }
    else {
      drupal_set_message($this->t('Successfully updated webhooks. %deleted were deleted, %updated were updated, and %created were created ', ['%deleted' => $deleted, '%updated' => $updated, '%created' => $created]));
    }
  }
}
